from flexmock import flexmock

from borgmatic.hooks import healthchecks as module


def mock_logger():
    logger = flexmock()
    logger.should_receive('addHandler')
    logger.should_receive('removeHandler')
    flexmock(module.logging).should_receive('getLogger').and_return(logger)


def test_initialize_monitor_creates_log_handler_with_ping_body_limit():
    ping_body_limit = 100
    monitoring_log_level = 1

    mock_logger()
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').with_args(
        module.HANDLER_IDENTIFIER,
        ping_body_limit - len(module.borgmatic.hooks.logs.PAYLOAD_TRUNCATION_INDICATOR),
        monitoring_log_level,
    ).once()

    module.initialize_monitor(
        {'ping_body_limit': ping_body_limit}, {}, 'test.yaml', monitoring_log_level, dry_run=False
    )


def test_initialize_monitor_creates_log_handler_with_default_ping_body_limit():
    monitoring_log_level = 1

    mock_logger()
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').with_args(
        module.HANDLER_IDENTIFIER,
        module.DEFAULT_PING_BODY_LIMIT_BYTES
        - len(module.borgmatic.hooks.logs.PAYLOAD_TRUNCATION_INDICATOR),
        monitoring_log_level,
    ).once()

    module.initialize_monitor({}, {}, 'test.yaml', monitoring_log_level, dry_run=False)


def test_initialize_monitor_creates_log_handler_with_zero_ping_body_limit():
    ping_body_limit = 0
    monitoring_log_level = 1

    mock_logger()
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').with_args(
        module.HANDLER_IDENTIFIER, ping_body_limit, monitoring_log_level
    ).once()

    module.initialize_monitor(
        {'ping_body_limit': ping_body_limit}, {}, 'test.yaml', monitoring_log_level, dry_run=False
    )


def test_initialize_monitor_creates_log_handler_when_send_logs_true():
    mock_logger()
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').once()

    module.initialize_monitor(
        {'send_logs': True}, {}, 'test.yaml', monitoring_log_level=1, dry_run=False
    )


def test_initialize_monitor_bails_when_send_logs_false():
    mock_logger()
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()

    module.initialize_monitor(
        {'send_logs': False}, {}, 'test.yaml', monitoring_log_level=1, dry_run=False
    )


def test_ping_monitor_hits_ping_url_for_start_state():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com'}
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/start', data=''.encode('utf-8'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_hits_ping_url_for_finish_state():
    hook_config = {'ping_url': 'https://example.com'}
    payload = 'data'
    flexmock(module.borgmatic.hooks.logs).should_receive('get_handler')
    flexmock(module.borgmatic.hooks.logs).should_receive(
        'format_buffered_logs_for_payload'
    ).and_return(payload)
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com', data=payload.encode('utf-8'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.FINISH,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_hits_ping_url_for_fail_state():
    hook_config = {'ping_url': 'https://example.com'}
    payload = 'data'
    flexmock(module.borgmatic.hooks.logs).should_receive('get_handler')
    flexmock(module.borgmatic.hooks.logs).should_receive(
        'format_buffered_logs_for_payload'
    ).and_return(payload)
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/fail', data=payload.encode('utf'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.FAIL,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_hits_ping_url_for_log_state():
    hook_config = {'ping_url': 'https://example.com'}
    payload = 'data'
    flexmock(module.borgmatic.hooks.logs).should_receive('get_handler')
    flexmock(module.borgmatic.hooks.logs).should_receive(
        'format_buffered_logs_for_payload'
    ).and_return(payload)
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/log', data=payload.encode('utf'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.LOG,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_with_ping_uuid_hits_corresponding_url():
    hook_config = {'ping_url': 'abcd-efgh-ijkl-mnop'}
    payload = 'data'
    flexmock(module.borgmatic.hooks.logs).should_receive('get_handler')
    flexmock(module.borgmatic.hooks.logs).should_receive(
        'format_buffered_logs_for_payload'
    ).and_return(payload)
    flexmock(module.requests).should_receive('post').with_args(
        f"https://hc-ping.com/{hook_config['ping_url']}",
        data=payload.encode('utf-8'),
        verify=True,
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.FINISH,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_skips_ssl_verification_when_verify_tls_false():
    hook_config = {'ping_url': 'https://example.com', 'verify_tls': False}
    payload = 'data'
    flexmock(module.borgmatic.hooks.logs).should_receive('get_handler')
    flexmock(module.borgmatic.hooks.logs).should_receive(
        'format_buffered_logs_for_payload'
    ).and_return(payload)
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com', data=payload.encode('utf-8'), verify=False
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.FINISH,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_executes_ssl_verification_when_verify_tls_true():
    hook_config = {'ping_url': 'https://example.com', 'verify_tls': True}
    payload = 'data'
    flexmock(module.borgmatic.hooks.logs).should_receive('get_handler')
    flexmock(module.borgmatic.hooks.logs).should_receive(
        'format_buffered_logs_for_payload'
    ).and_return(payload)
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com', data=payload.encode('utf-8'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.FINISH,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_dry_run_does_not_hit_ping_url():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com'}
    flexmock(module.requests).should_receive('post').never()

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=True,
    )


def test_ping_monitor_does_not_hit_ping_url_when_states_not_matching():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com', 'states': ['finish']}
    flexmock(module.requests).should_receive('post').never()

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=True,
    )


def test_ping_monitor_hits_ping_url_when_states_matching():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com', 'states': ['start', 'finish']}
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/start', data=''.encode('utf-8'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_adds_create_query_parameter_when_create_slug_true():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com', 'create_slug': True}
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/start?create=1', data=''.encode('utf-8'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_does_not_add_create_query_parameter_when_create_slug_false():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com', 'create_slug': False}
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/start', data=''.encode('utf-8'), verify=True
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_does_not_add_create_query_parameter_when_ping_url_is_uuid():
    hook_config = {'ping_url': 'b3611b24-df9c-4d36-9203-fa292820bf2a', 'create_slug': True}
    flexmock(module.requests).should_receive('post').with_args(
        f"https://hc-ping.com/{hook_config['ping_url']}",
        data=''.encode('utf-8'),
        verify=True,
    ).and_return(flexmock(ok=True))

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.FINISH,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_issues_warning_when_ping_url_is_uuid_and_create_slug_true():
    hook_config = {'ping_url': 'b3611b24-df9c-4d36-9203-fa292820bf2a', 'create_slug': True}

    flexmock(module.requests).should_receive('post').and_return(flexmock(ok=True))

    flexmock(module.logger).should_receive('warning').once()

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.FINISH,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_with_connection_error_logs_warning():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com'}
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/start', data=''.encode('utf-8'), verify=True
    ).and_raise(module.requests.exceptions.ConnectionError)
    flexmock(module.logger).should_receive('warning').once()

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=False,
    )


def test_ping_monitor_with_other_error_logs_warning():
    flexmock(module.borgmatic.hooks.logs).should_receive('Forgetful_buffering_handler').never()
    hook_config = {'ping_url': 'https://example.com'}
    response = flexmock(ok=False)
    response.should_receive('raise_for_status').and_raise(
        module.requests.exceptions.RequestException
    )
    flexmock(module.requests).should_receive('post').with_args(
        'https://example.com/start', data=''.encode('utf-8'), verify=True
    ).and_return(response)
    flexmock(module.logger).should_receive('warning').once()

    module.ping_monitor(
        hook_config,
        {},
        'config.yaml',
        state=module.monitor.State.START,
        monitoring_log_level=1,
        dry_run=False,
    )



import pytest
import ruamel.yaml
from flexmock import flexmock

from borgmatic.config import override as module


def test_set_values_with_empty_keys_bails():
    config = {}

    module.set_values(config, keys=(), value='value')

    assert config == {}


def test_set_values_with_one_key_sets_it_into_config():
    config = {}

    module.set_values(config, keys=('key',), value='value')

    assert config == {'key': 'value'}


def test_set_values_with_one_key_overwrites_existing_key():
    config = {'key': 'old_value', 'other': 'other_value'}

    module.set_values(config, keys=('key',), value='value')

    assert config == {'key': 'value', 'other': 'other_value'}


def test_set_values_with_multiple_keys_creates_hierarchy():
    config = {}

    module.set_values(config, ('option', 'suboption'), 'value')

    assert config == {'option': {'suboption': 'value'}}


def test_set_values_with_multiple_keys_updates_hierarchy():
    config = {'option': {'other': 'other_value'}}
    module.set_values(config, ('option', 'key'), 'value')

    assert config == {'option': {'key': 'value', 'other': 'other_value'}}


def test_set_values_with_key_when_list_index_expected_errors():
    config = {'option': ['foo', 'bar', 'baz']}

    with pytest.raises(ValueError):
        module.set_values(config, keys=('option', 'key'), value='value')


@pytest.mark.parametrize(
    'schema,option_keys,expected_type',
    (
        ({'properties': {'foo': {'type': 'array'}}}, ('foo',), 'array'),
        (
            {'properties': {'foo': {'properties': {'bar': {'type': 'array'}}}}},
            ('foo', 'bar'),
            'array',
        ),
        ({'properties': {'foo': {'type': 'array'}}}, ('other',), None),
        ({'properties': {'foo': {'description': 'stuff'}}}, ('foo',), None),
        ({}, ('foo',), None),
    ),
)
def test_type_for_option_grabs_type_if_found_in_schema(schema, option_keys, expected_type):
    assert module.type_for_option(schema, option_keys) == expected_type


@pytest.mark.parametrize(
    'key,expected_key',
    (
        (('foo', 'bar'), ('foo', 'bar')),
        (('location', 'foo'), ('foo',)),
        (('storage', 'foo'), ('foo',)),
        (('retention', 'foo'), ('foo',)),
        (('consistency', 'foo'), ('foo',)),
        (('output', 'foo'), ('foo',)),
        (('hooks', 'foo', 'bar'), ('foo', 'bar')),
        (('foo', 'hooks'), ('foo', 'hooks')),
    ),
)
def test_strip_section_names_passes_through_key_without_section_name(key, expected_key):
    assert module.strip_section_names(key) == expected_key


def test_parse_overrides_splits_keys_and_values():
    flexmock(module).should_receive('strip_section_names').replace_with(lambda value: value)
    flexmock(module).should_receive('type_for_option').and_return('string')
    flexmock(module).should_receive('convert_value_type').replace_with(
        lambda value, option_type: value
    )
    raw_overrides = ['option.my_option=value1', 'other_option=value2']
    expected_result = (
        (('option', 'my_option'), 'value1'),
        (('other_option'), 'value2'),
    )

    module.parse_overrides(raw_overrides, schema={}) == expected_result


def test_parse_overrides_allows_value_with_equal_sign():
    flexmock(module).should_receive('strip_section_names').replace_with(lambda value: value)
    flexmock(module).should_receive('type_for_option').and_return('string')
    flexmock(module).should_receive('convert_value_type').replace_with(
        lambda value, option_type: value
    )
    raw_overrides = ['option=this===value']
    expected_result = ((('option',), 'this===value'),)

    module.parse_overrides(raw_overrides, schema={}) == expected_result


def test_parse_overrides_raises_on_missing_equal_sign():
    flexmock(module).should_receive('strip_section_names').replace_with(lambda value: value)
    flexmock(module).should_receive('type_for_option').and_return('string')
    flexmock(module).should_receive('convert_value_type').replace_with(
        lambda value, option_type: value
    )
    raw_overrides = ['option']

    with pytest.raises(ValueError):
        module.parse_overrides(raw_overrides, schema={})


def test_parse_overrides_raises_on_invalid_override_value():
    flexmock(module).should_receive('strip_section_names').replace_with(lambda value: value)
    flexmock(module).should_receive('type_for_option').and_return('string')
    flexmock(module).should_receive('convert_value_type').and_raise(ruamel.yaml.parser.ParserError)
    raw_overrides = ['option=[in valid]']

    with pytest.raises(ValueError):
        module.parse_overrides(raw_overrides, schema={})


def test_parse_overrides_allows_value_with_single_key():
    flexmock(module).should_receive('strip_section_names').replace_with(lambda value: value)
    flexmock(module).should_receive('type_for_option').and_return('string')
    flexmock(module).should_receive('convert_value_type').replace_with(
        lambda value, option_type: value
    )
    raw_overrides = ['option=value']
    expected_result = ((('option',), 'value'),)

    module.parse_overrides(raw_overrides, schema={}) == expected_result


def test_parse_overrides_handles_empty_overrides():
    module.parse_overrides(raw_overrides=None, schema={}) == ()



import logging
import os
import shlex

from borgmatic.execute import execute_command, execute_command_with_processes
from borgmatic.hooks import dump

logger = logging.getLogger(__name__)


def make_dump_path(config):  # pragma: no cover
    '''
    Make the dump path from the given configuration dict and the name of this hook.
    '''
    return dump.make_data_source_dump_path(
        config.get('borgmatic_source_directory'), 'sqlite_databases'
    )


def use_streaming(databases, config, log_prefix):
    '''
    Given a sequence of SQLite database configuration dicts, a configuration dict (ignored), and a
    log prefix (ignored), return whether streaming will be using during dumps.
    '''
    return any(databases)


def dump_data_sources(databases, config, log_prefix, dry_run):
    '''
    Dump the given SQLite databases to a named pipe. The databases are supplied as a sequence of
    configuration dicts, as per the configuration schema. Use the given configuration dict to
    construct the destination path and the given log prefix in any log entries.

    Return a sequence of subprocess.Popen instances for the dump processes ready to spew to a named
    pipe. But if this is a dry run, then don't actually dump anything and return an empty sequence.
    '''
    dry_run_label = ' (dry run; not actually dumping anything)' if dry_run else ''
    processes = []

    logger.info(f'{log_prefix}: Dumping SQLite databases{dry_run_label}')

    for database in databases:
        database_path = database['path']

        if database['name'] == 'all':
            logger.warning('The "all" database name has no meaning for SQLite databases')
        if not os.path.exists(database_path):
            logger.warning(
                f'{log_prefix}: No SQLite database at {database_path}; an empty database will be created and dumped'
            )

        dump_path = make_dump_path(config)
        dump_filename = dump.make_data_source_dump_filename(dump_path, database['name'])

        if os.path.exists(dump_filename):
            logger.warning(
                f'{log_prefix}: Skipping duplicate dump of SQLite database at {database_path} to {dump_filename}'
            )
            continue

        command = (
            'sqlite3',
            shlex.quote(database_path),
            '.dump',
            '>',
            shlex.quote(dump_filename),
        )
        logger.debug(
            f'{log_prefix}: Dumping SQLite database at {database_path} to {dump_filename}{dry_run_label}'
        )
        if dry_run:
            continue

        dump.create_named_pipe_for_dump(dump_filename)
        processes.append(execute_command(command, shell=True, run_to_completion=False))

    return processes


def remove_data_source_dumps(databases, config, log_prefix, dry_run):  # pragma: no cover
    '''
    Remove the given SQLite database dumps from the filesystem. The databases are supplied as a
    sequence of configuration dicts, as per the configuration schema. Use the given configuration
    dict to construct the destination path and the given log prefix in any log entries. If this is a
    dry run, then don't actually remove anything.
    '''
    dump.remove_data_source_dumps(make_dump_path(config), 'SQLite', log_prefix, dry_run)


def make_data_source_dump_pattern(databases, config, log_prefix, name=None):  # pragma: no cover
    '''
    Make a pattern that matches the given SQLite databases. The databases are supplied as a sequence
    of configuration dicts, as per the configuration schema.
    '''
    return dump.make_data_source_dump_filename(make_dump_path(config), name)


def restore_data_source_dump(
    hook_config, config, log_prefix, data_source, dry_run, extract_process, connection_params
):
    '''
    Restore a database from the given extract stream. The database is supplied as a data source
    configuration dict, but the given hook configuration is ignored. The given configuration dict is
    used to construct the destination path, and the given log prefix is used for any log entries. If
    this is a dry run, then don't actually restore anything. Trigger the given active extract
    process (an instance of subprocess.Popen) to produce output to consume.
    '''
    dry_run_label = ' (dry run; not actually restoring anything)' if dry_run else ''
    database_path = connection_params['restore_path'] or data_source.get(
        'restore_path', data_source.get('path')
    )

    logger.debug(f'{log_prefix}: Restoring SQLite database at {database_path}{dry_run_label}')
    if dry_run:
        return

    try:
        os.remove(database_path)
        logger.warning(f'{log_prefix}: Removed existing SQLite database at {database_path}')
    except FileNotFoundError:  # pragma: no cover
        pass

    restore_command = (
        'sqlite3',
        database_path,
    )

    # Don't give Borg local path so as to error on warnings, as "borg extract" only gives a warning
    # if the restore paths don't exist in the archive.
    execute_command_with_processes(
        restore_command,
        [extract_process],
        output_log_level=logging.DEBUG,
        input_file=extract_process.stdout,
    )



import logging

import borgmatic.borg.environment
import borgmatic.borg.feature
import borgmatic.borg.flags
import borgmatic.execute

logger = logging.getLogger(__name__)


def make_rdelete_command(
    repository,
    config,
    local_borg_version,
    rdelete_arguments,
    global_arguments,
    local_path,
    remote_path,
):
    '''
    Given a local or remote repository dict, a configuration dict, the local Borg version, the
    arguments to the rdelete action as an argparse.Namespace, and global arguments, return a command
    as a tuple to rdelete the entire repository.
    '''
    return (
        (local_path,)
        + (
            ('rdelete',)
            if borgmatic.borg.feature.available(
                borgmatic.borg.feature.Feature.RDELETE, local_borg_version
            )
            else ('delete',)
        )
        + (('--info',) if logger.getEffectiveLevel() == logging.INFO else ())
        + (('--debug', '--show-rc') if logger.isEnabledFor(logging.DEBUG) else ())
        + borgmatic.borg.flags.make_flags('dry-run', global_arguments.dry_run)
        + borgmatic.borg.flags.make_flags('remote-path', remote_path)
        + borgmatic.borg.flags.make_flags('log-json', global_arguments.log_json)
        + borgmatic.borg.flags.make_flags('lock-wait', config.get('lock_wait'))
        + borgmatic.borg.flags.make_flags('list', rdelete_arguments.list_archives)
        + (
            (('--force',) + (('--force',) if rdelete_arguments.force >= 2 else ()))
            if rdelete_arguments.force
            else ()
        )
        + borgmatic.borg.flags.make_flags_from_arguments(
            rdelete_arguments, excludes=('list_archives', 'force', 'repository')
        )
        + borgmatic.borg.flags.make_repository_flags(repository['path'], local_borg_version)
    )


def delete_repository(
    repository,
    config,
    local_borg_version,
    rdelete_arguments,
    global_arguments,
    local_path='borg',
    remote_path=None,
):
    '''
    Given a local or remote repository dict, a configuration dict, the local Borg version, the
    arguments to the rdelete action as an argparse.Namespace, global arguments as an
    argparse.Namespace, and local and remote Borg paths, rdelete the entire repository.
    '''
    borgmatic.logger.add_custom_log_levels()

    command = make_rdelete_command(
        repository,
        config,
        local_borg_version,
        rdelete_arguments,
        global_arguments,
        local_path,
        remote_path,
    )

    borgmatic.execute.execute_command(
        command,
        output_log_level=logging.ANSWER,
        # Don't capture output when Borg is expected to prompt for interactive confirmation, or the
        # prompt won't work.
        output_file=(
            None
            if rdelete_arguments.force or rdelete_arguments.cache_only
            else borgmatic.execute.DO_NOT_CAPTURE
        ),
        extra_environment=borgmatic.borg.environment.make_environment(config),
        borg_local_path=local_path,
        borg_exit_codes=config.get('borg_exit_codes'),
    )



from flexmock import flexmock

from borgmatic.actions import rcreate as module


def test_run_rcreate_does_not_raise():
    flexmock(module.logger).answer = lambda message: None
    flexmock(module.borgmatic.config.validate).should_receive('repositories_match').and_return(True)
    flexmock(module.borgmatic.borg.rcreate).should_receive('create_repository')
    arguments = flexmock(
        encryption_mode=flexmock(),
        source_repository=flexmock(),
        repository=flexmock(),
        copy_crypt_key=flexmock(),
        append_only=flexmock(),
        storage_quota=flexmock(),
        make_parent_dirs=flexmock(),
    )

    module.run_rcreate(
        repository={'path': 'repo'},
        config={},
        local_borg_version=None,
        rcreate_arguments=arguments,
        global_arguments=flexmock(dry_run=False),
        local_path=None,
        remote_path=None,
    )


def test_run_rcreate_bails_if_repository_does_not_match():
    flexmock(module.logger).answer = lambda message: None
    flexmock(module.borgmatic.config.validate).should_receive('repositories_match').and_return(
        False
    )
    flexmock(module.borgmatic.borg.rcreate).should_receive('create_repository').never()
    arguments = flexmock(
        encryption_mode=flexmock(),
        source_repository=flexmock(),
        repository=flexmock(),
        copy_crypt_key=flexmock(),
        append_only=flexmock(),
        storage_quota=flexmock(),
        make_parent_dirs=flexmock(),
    )

    module.run_rcreate(
        repository={'path': 'repo'},
        config={},
        local_borg_version=None,
        rcreate_arguments=arguments,
        global_arguments=flexmock(dry_run=False),
        local_path=None,
        remote_path=None,
    )



import logging

from flexmock import flexmock

from borgmatic.borg import rdelete as module

from ..test_verbosity import insert_logging_mock


def test_make_rdelete_command_with_feature_available_runs_borg_rdelete():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', 'repo')


def test_make_rdelete_command_without_feature_available_runs_borg_delete():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(False)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'delete', 'repo')


def test_make_rdelete_command_includes_log_info():
    insert_logging_mock(logging.INFO)
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--info', 'repo')


def test_make_rdelete_command_includes_log_debug():
    insert_logging_mock(logging.DEBUG)
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--debug', '--show-rc', 'repo')


def test_make_rdelete_command_includes_dry_run():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').with_args(
        'dry-run', True
    ).and_return(('--dry-run',))
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=True, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--dry-run', 'repo')


def test_make_rdelete_command_includes_remote_path():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').with_args(
        'remote-path', 'borg1'
    ).and_return(('--remote-path', 'borg1'))
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path='borg1',
    )

    assert command == ('borg', 'rdelete', '--remote-path', 'borg1', 'repo')


def test_make_rdelete_command_includes_log_json():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').with_args(
        'log-json', True
    ).and_return(('--log-json',))
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=False, log_json=True),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--log-json', 'repo')


def test_make_rdelete_command_includes_lock_wait():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').with_args(
        'lock-wait', 5
    ).and_return(('--lock-wait', '5'))
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={'lock_wait': 5},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=0),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--lock-wait', '5', 'repo')


def test_make_rdelete_command_includes_list():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').with_args(
        'list', True
    ).and_return(('--list',))
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=True, force=0),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--list', 'repo')


def test_make_rdelete_command_includes_force():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=1),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--force', 'repo')


def test_make_rdelete_command_includes_force_twice():
    flexmock(module.borgmatic.borg.feature).should_receive('available').and_return(True)
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_flags_from_arguments').and_return(())
    flexmock(module.borgmatic.borg.flags).should_receive('make_repository_flags').and_return(
        ('repo',)
    )

    command = module.make_rdelete_command(
        repository={'path': 'repo'},
        config={},
        local_borg_version='1.2.3',
        rdelete_arguments=flexmock(list_archives=False, force=2),
        global_arguments=flexmock(dry_run=False, log_json=False),
        local_path='borg',
        remote_path=None,
    )

    assert command == ('borg', 'rdelete', '--force', '--force', 'repo')


def test_delete_repository_with_defaults_does_not_capture_output():
    flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
    command = flexmock()
    flexmock(module).should_receive('make_rdelete_command').and_return(command)
    flexmock(module.borgmatic.borg.environment).should_receive('make_environment').and_return(
        flexmock()
    )
    flexmock(module.borgmatic.execute).should_receive('execute_command').with_args(
        command,
        output_log_level=module.logging.ANSWER,
        output_file=module.borgmatic.execute.DO_NOT_CAPTURE,
        extra_environment=object,
        borg_local_path='borg',
        borg_exit_codes=None,
    ).once()

    module.delete_repository(
        repository={'path': 'repo'},
        config={},
        local_borg_version=flexmock(),
        rdelete_arguments=flexmock(force=False, cache_only=False),
        global_arguments=flexmock(),
        local_path='borg',
        remote_path=None,
    )


def test_delete_repository_with_force_captures_output():
    flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
    command = flexmock()
    flexmock(module).should_receive('make_rdelete_command').and_return(command)
    flexmock(module.borgmatic.borg.environment).should_receive('make_environment').and_return(
        flexmock()
    )
    flexmock(module.borgmatic.execute).should_receive('execute_command').with_args(
        command,
        output_log_level=module.logging.ANSWER,
        output_file=None,
        extra_environment=object,
        borg_local_path='borg',
        borg_exit_codes=None,
    ).once()

    module.delete_repository(
        repository={'path': 'repo'},
        config={},
        local_borg_version=flexmock(),
        rdelete_arguments=flexmock(force=True, cache_only=False),
        global_arguments=flexmock(),
        local_path='borg',
        remote_path=None,
    )


def test_delete_repository_with_cache_only_captures_output():
    flexmock(module.borgmatic.logger).should_receive('add_custom_log_levels')
    command = flexmock()
    flexmock(module).should_receive('make_rdelete_command').and_return(command)
    flexmock(module.borgmatic.borg.environment).should_receive('make_environment').and_return(
        flexmock()
    )
    flexmock(module.borgmatic.execute).should_receive('execute_command').with_args(
        command,
        output_log_level=module.logging.ANSWER,
        output_file=None,
        extra_environment=object,
        borg_local_path='borg',
        borg_exit_codes=None,
    ).once()

    module.delete_repository(
        repository={'path': 'repo'},
        config={},
        local_borg_version=flexmock(),
        rdelete_arguments=flexmock(force=False, cache_only=True),
        global_arguments=flexmock(),
        local_path='borg',
        remote_path=None,
    )



from borgmatic.commands.completion import actions as module


def test_upgrade_message_does_not_raise():
    module.upgrade_message(
        language='English', upgrade_command='read a lot', completion_file='your brain'
    )



import logging

import borgmatic.borg.rdelete

logger = logging.getLogger(__name__)


def run_rdelete(
    repository,
    config,
    local_borg_version,
    rdelete_arguments,
    global_arguments,
    local_path,
    remote_path,
):
    '''
    Run the "rdelete" action for the given repository.
    '''
    if rdelete_arguments.repository is None or borgmatic.config.validate.repositories_match(
        repository, rdelete_arguments.repository
    ):
        logger.answer(
            f'{repository.get("label", repository["path"])}: Deleting repository'
            + (' cache' if rdelete_arguments.cache_only else '')
        )

        borgmatic.borg.rdelete.delete_repository(
            repository,
            config,
            local_borg_version,
            rdelete_arguments,
            global_arguments,
            local_path,
            remote_path,
        )
